@@ -4,16 +4,19 @@ from enum import Enum
 from typing import Mapping, Optional, Union
 
 from furl import furl
-from mbtest.imposters.base import JsonSerializable, JsonStructure
+
+from mbtest.imposters.base import Injecting, JsonSerializable, JsonStructure
 
 
 class BasePredicate(JsonSerializable, metaclass=ABCMeta):
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "BasePredicate":
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "BasePredicate":  # noqa: C901
         if "and" in structure:
             return AndPredicate.from_structure(structure)
         elif "or" in structure:
             return OrPredicate.from_structure(structure)
+        elif "not" in structure:
+            return NotPredicate.from_structure(structure)
         elif "contains" in structure and "data" in structure["contains"]:
             return TcpPredicate.from_structure(structure)
         elif "inject" in structure:
@@ -32,9 +35,12 @@ class LogicallyCombinablePredicate(BasePredicate, metaclass=ABCMeta):
     def __or__(self, other: "BasePredicate") -> "OrPredicate":
         return OrPredicate(self, other)
 
+    def __invert__(self) -> "NotPredicate":
+        return NotPredicate(self)
+
 
 class Predicate(LogicallyCombinablePredicate):
-    """ Represents a `Mountebank predicate <http://www.mbtest.org/docs/api/predicates>`_.
+    """Represents a `Mountebank predicate <http://www.mbtest.org/docs/api/predicates>`_.
     A predicate can be thought of as a trigger, which may or may not match a request.
 
     :param path: URL path.
@@ -58,6 +64,7 @@ class Predicate(LogicallyCombinablePredicate):
         HEAD = "HEAD"
         POST = "POST"
         PUT = "PUT"
+        PATCH = "PATCH"
 
     class Operator(Enum):
         """`Predicate operator <http://www.mbtest.org/docs/api/predicates>`_."""
@@ -104,43 +111,41 @@ class Predicate(LogicallyCombinablePredicate):
 
     def as_structure(self) -> JsonStructure:
         predicate = {
-            self.operator.value: self._fields_as_structure(),
+            self.operator.value: self.fields_as_structure(),
             "caseSensitive": self.case_sensitive,
         }
         if self.xpath:
             predicate["xpath"] = {"selector": self.xpath}
         return predicate
 
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "Predicate":
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "Predicate":
         operators = tuple(filter(Predicate.Operator.has_value, structure.keys()))
         if len(operators) != 1:
             raise Predicate.InvalidPredicateOperator(
                 "Each predicate must define exactly one operator."
             )
         operator = operators[0]
-        predicate = Predicate(
-            operator=operator, case_sensitive=structure.get("caseSensitive", True)
-        )
-        predicate._fields_from_structure(structure[operator])
+        predicate = cls(operator=operator, case_sensitive=structure.get("caseSensitive", True))
+        predicate.fields_from_structure(structure[operator])
         if "xpath" in structure:
             predicate.xpath = structure["xpath"]["selector"]
         return predicate
 
-    def _fields_from_structure(self, inner):
-        self._set_if_in_dict(inner, "path", "path")
-        self._set_if_in_dict(inner, "query", "query")
-        self._set_if_in_dict(inner, "body", "body")
-        self._set_if_in_dict(inner, "headers", "headers")
+    def fields_from_structure(self, inner):
+        self.set_if_in_dict(inner, "path", "path")
+        self.set_if_in_dict(inner, "query", "query")
+        self.set_if_in_dict(inner, "body", "body")
+        self.set_if_in_dict(inner, "headers", "headers")
         if "method" in inner:
             self.method = Predicate.Method(inner["method"])
 
-    def _fields_as_structure(self):
+    def fields_as_structure(self):
         fields = {}
-        self._add_if_true(fields, "path", self.path)
-        self._add_if_true(fields, "query", self.query)
-        self._add_if_true(fields, "body", self.body)
-        self._add_if_true(fields, "headers", self.headers)
+        self.add_if_true(fields, "path", self.path)
+        self.add_if_true(fields, "query", self.query)
+        self.add_if_true(fields, "body", self.body)
+        self.add_if_true(fields, "headers", self.headers)
         if self.method:
             fields["method"] = self.method.value
         return fields
@@ -154,9 +159,9 @@ class AndPredicate(LogicallyCombinablePredicate):
     def as_structure(self) -> JsonStructure:
         return {"and": [self.left.as_structure(), self.right.as_structure()]}
 
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "AndPredicate":
-        return AndPredicate(
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "AndPredicate":
+        return cls(
             BasePredicate.from_structure(structure["and"][0]),
             BasePredicate.from_structure(structure["and"][1]),
         )
@@ -170,14 +175,26 @@ class OrPredicate(LogicallyCombinablePredicate):
     def as_structure(self) -> JsonStructure:
         return {"or": [self.left.as_structure(), self.right.as_structure()]}
 
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "OrPredicate":
-        return OrPredicate(
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "OrPredicate":
+        return cls(
             BasePredicate.from_structure(structure["or"][0]),
             BasePredicate.from_structure(structure["or"][1]),
         )
 
 
+class NotPredicate(LogicallyCombinablePredicate):
+    def __init__(self, inverted: BasePredicate) -> None:
+        self.inverted = inverted
+
+    def as_structure(self) -> JsonStructure:
+        return {"not": self.inverted.as_structure()}
+
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "NotPredicate":
+        return cls(BasePredicate.from_structure(structure["not"]))
+
+
 class TcpPredicate(LogicallyCombinablePredicate):
     """Represents a `Mountebank TCP predicate <http://www.mbtest.org/docs/protocols/tcp>`_.
     A predicate can be thought of as a trigger, which may or may not match a request.
@@ -191,12 +208,12 @@ class TcpPredicate(LogicallyCombinablePredicate):
     def as_structure(self) -> JsonStructure:
         return {"contains": {"data": self.data}}
 
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "TcpPredicate":
-        return TcpPredicate(structure["contains"]["data"])
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "TcpPredicate":
+        return cls(structure["contains"]["data"])
 
 
-class InjectionPredicate(BasePredicate):
+class InjectionPredicate(BasePredicate, Injecting):
     """Represents a `Mountebank injection predicate <http://www.mbtest.org/docs/api/injection>`_.
     A predicate can be thought of as a trigger, which may or may not match a request.
 
@@ -205,12 +222,6 @@ class InjectionPredicate(BasePredicate):
     :param inject: JavaScript function to inject.
     """
 
-    def __init__(self, inject: str) -> None:
-        self.inject = inject
-
-    def as_structure(self) -> JsonStructure:
-        return {"inject": self.inject}
-
-    @staticmethod
-    def from_structure(structure: JsonStructure) -> "InjectionPredicate":
-        return InjectionPredicate(inject=structure["inject"])
+    @classmethod
+    def from_structure(cls, structure: JsonStructure) -> "InjectionPredicate":
+        return cls(inject=structure["inject"])
