@@ -1,6 +1,6 @@
 # encoding=utf-8
 import warnings
-from typing import Any, Mapping, Union
+from typing import Any, Mapping, Sequence, Union, cast
 
 from furl import furl
 from hamcrest import anything
@@ -9,8 +9,9 @@ from hamcrest.core.core.isanything import IsAnything
 from hamcrest.core.description import Description
 from hamcrest.core.helpers.wrap_matcher import wrap_matcher
 from hamcrest.core.matcher import Matcher
+
+from mbtest.imposters.imposters import HttpRequest, Imposter, SentEmail
 from mbtest.server import MountebankServer
-from more_itertools import flatten
 
 ANYTHING = anything()
 
@@ -22,7 +23,7 @@ def had_request(
     headers: Union[Mapping[str, str], Matcher[Mapping[str, str]]] = ANYTHING,
     body: Union[str, Matcher[str]] = ANYTHING,
     times: Union[int, Matcher[int]] = ANYTHING,
-) -> Matcher[MountebankServer]:
+) -> Matcher[Union[Imposter, MountebankServer]]:
     """Mountebank server has recorded call matching.
 
     Build criteria with `with_` and `and_` methods:
@@ -72,12 +73,12 @@ class HadRequest(BaseMatcher):
             or times != ANYTHING
         ):  # pragma: no cover
             warnings.warn("Use builder-style with_X and and_X methods, rather than arguments.")
-        self.method = wrap_matcher(method)  # type: Matcher[str]
-        self.path = wrap_matcher(path)  # type: Matcher[Union[furl, str]]
-        self.query = wrap_matcher(query)  # type Matcher[Mapping[str, str]]
-        self.headers = wrap_matcher(headers)  # type Matcher[Mapping[str, str]]
-        self.body = wrap_matcher(body)  # type: Matcher[str]
-        self.times = wrap_matcher(times)  # type: Matcher[int]
+        self.method: Matcher[str] = wrap_matcher(method)
+        self.path: Matcher[Union[furl, str]] = wrap_matcher(path)
+        self.query: Matcher[Mapping[str, str]] = wrap_matcher(query)
+        self.headers: Matcher[Mapping[str, str]] = wrap_matcher(headers)
+        self.body: Matcher[str] = wrap_matcher(body)
+        self.times: Matcher[int] = wrap_matcher(times)
 
     def describe_to(self, description: Description) -> None:
         if isinstance(self.times, IsAnything):
@@ -99,27 +100,27 @@ class HadRequest(BaseMatcher):
         field_matcher: Matcher[Any], field_name: str, description: Description
     ) -> None:
         if not isinstance(field_matcher, IsAnything):
-            description.append_text(" {0}: ".format(field_name)).append_description_of(
-                field_matcher
-            )
+            description.append_text(f" {field_name}: ").append_description_of(field_matcher)
 
-    def describe_mismatch(self, server: MountebankServer, description: Description) -> None:
+    def describe_mismatch(
+        self, actual: Union[Imposter, MountebankServer], description: Description
+    ) -> None:
         description.append_text("found ").append_description_of(len(self.matching_requests))
         description.append_text(" matching requests: ").append_description_of(
             self.matching_requests
         )
         description.append_text(". All requests: ").append_description_of(self.all_requests)
 
-    def _matches(self, server: MountebankServer) -> bool:
-        self.all_requests = list(flatten(server.get_actual_requests().values()))
+    def _matches(self, actual: Union[Imposter, MountebankServer]) -> bool:
+        self.all_requests = cast(Sequence[HttpRequest], list(actual.get_actual_requests()))
         self.matching_requests = [
             request
             for request in self.all_requests
-            if self.method.matches(request.get("method", None))
-            and self.path.matches(request.get("path", None))
-            and self.query.matches(request.get("query", None))
-            and self.headers.matches(request.get("headers", None))
-            and self.body.matches(request.get("body", None))
+            if self.method.matches(request.method)
+            and self.path.matches(request.path)
+            and self.query.matches(request.query)
+            and self.headers.matches(request.headers)
+            and self.body.matches(request.body)
         ]
 
         if isinstance(self.times, IsAnything):
@@ -174,7 +175,7 @@ def email_sent(
     to: Union[str, Matcher[str]] = ANYTHING,
     subject: Union[str, Matcher[str]] = ANYTHING,
     body_text: Union[str, Matcher[str]] = ANYTHING,
-) -> Matcher[MountebankServer]:
+) -> Matcher[Union[Imposter, MountebankServer]]:
     """Mountebank SMTP server was asked to sent email matching:
 
     :param to: Email's to field matched...
@@ -198,9 +199,12 @@ class EmailSent(BaseMatcher):
         subject: Union[str, Matcher[str]] = ANYTHING,
         body_text: Union[str, Matcher[str]] = ANYTHING,
     ) -> None:
+        # TODO: builder style, & lots more attributes.
         self.body_text = wrap_matcher(body_text)
         self.subject = wrap_matcher(subject)
-        self.to = wrap_matcher(to)
+        self.to = wrap_matcher(
+            to
+        )  # TODO: Type is wrong here - to is a sequence of addresses and names.
 
     def describe_to(self, description: Description) -> None:
         description.append_text("email with")
@@ -214,24 +218,33 @@ class EmailSent(BaseMatcher):
     @staticmethod
     def _append_matcher_description(description: Description, matcher: Matcher, text: str) -> None:
         if not isinstance(matcher, IsAnything):
-            description.append_text(" {0}: ".format(text)).append_description_of(matcher)
+            description.append_text(f" {text}: ").append_description_of(matcher)
 
-    def describe_mismatch(self, server: MountebankServer, description: Description) -> None:
-        description.append_text("found ").append_description_of(len(self.matching_requests))
-        description.append_text(" matching requests: ").append_description_of(
-            self.matching_requests
-        )
-        description.append_text(". All requests: ").append_description_of(self.all_requests)
+    def describe_mismatch(
+        self, actual: Union[Imposter, MountebankServer], description: Description
+    ) -> None:
+        sent_email = self.get_sent_email(actual)
+        matching_emails = self.get_matching_emails(sent_email)
 
-    def _matches(self, server: MountebankServer) -> bool:
-        self.all_requests = list(flatten(server.get_actual_requests().values()))
-        self.matching_requests = [
-            request
-            for request in self.all_requests
-            if "envelopeFrom" in request
-            and self.body_text.matches(request.get("text", None))
-            and self.subject.matches(request.get("subject", None))
-            and self.to.matches(request.get("to", None))
+        description.append_text("found ").append_description_of(len(matching_emails))
+        description.append_text(" matching emails: ").append_description_of(matching_emails)
+        description.append_text(". All emails: ").append_description_of(sent_email)
+
+    def _matches(self, actual: Union[Imposter, MountebankServer]) -> bool:
+        sent_email = self.get_sent_email(actual)
+        matching_emails = self.get_matching_emails(sent_email)
+
+        return len(matching_emails) > 0
+
+    @staticmethod
+    def get_sent_email(actual) -> Sequence[SentEmail]:
+        return cast(Sequence[SentEmail], list(actual.get_actual_requests()))
+
+    def get_matching_emails(self, sent_email) -> Sequence[SentEmail]:
+        return [
+            email
+            for email in sent_email
+            if self.body_text.matches(email.text)
+            and self.subject.matches(email.subject)
+            and self.to.matches(email.to)
         ]
-
-        return len(self.matching_requests) > 0
