@@ -1,20 +1,21 @@
 ﻿# encoding=utf-8
-import collections.abc as abc
 import logging
 import platform
 import subprocess  # nosec
 import time
+from collections import abc
 from pathlib import Path
 from threading import Lock
-from typing import Iterable, List, Mapping, MutableMapping, Sequence, Set, Union, cast
+from typing import Iterable, Iterator, List, MutableSequence, Sequence, Set, Union
 
 import requests
 from _pytest.fixtures import FixtureRequest  # type: ignore
 from furl import furl
-from mbtest.imposters import Imposter
-from mbtest.imposters.base import JsonStructure
 from requests import RequestException
 
+from mbtest.imposters import Imposter
+from mbtest.imposters.imposters import Request
+
 DEFAULT_MB_EXECUTABLE = str(
     Path("node_modules") / ".bin" / ("mb.cmd" if platform.system() == "Windows" else "mb")
 )
@@ -33,7 +34,7 @@ def mock_server(
     data_dir: Union[str, None] = ".mbdb",
 ) -> "ExecutingMountebankServer":
     """`Pytest fixture <https://docs.pytest.org/en/latest/fixture.html>`_, making available a mock server, running one
-    or more impostors, one for each domain being mocked.
+    or more imposters, one for each domain being mocked.
 
     Use in a pytest conftest.py fixture as follows::
 
@@ -49,7 +50,7 @@ def mock_server(
                                 record_requests=True)
 
             with mock_server(imposter) as s:
-                r = requests.get('{0}/test'.format(imposter.url))
+                r = requests.get(f"{imposter.url}/test")
 
                 assert_that(r, is_response().with_status_code(200).and_body("sausages"))
                 assert_that(s, had_request(path='/test', method="GET"))
@@ -96,18 +97,18 @@ class MountebankServer:
                                      Response(body='sausages')),
                                 record_requests=True)
 
-            with mb(imposter) as s:
-                r = requests.get('{0}/test'.format(imposter.url))
+            with mb(imposter):
+                r = requests.get(f"{imposter.url}/test")
 
                 assert_that(r, is_response().with_status_code(200).and_body("sausages"))
-                assert_that(s, had_request(path='/test', method="GET"))
+                assert_that(imposter, had_request(path='/test', method="GET"))
 
-    Impostors will be torn down when the `with` block is exited.
+    Imposters will be torn down when the `with` block is exited.
 
     :param port: Server port.
     :param scheme: Server scheme, if not `http`.
     :param host: Server host, if not `localhost`.
-    :param imposters_path: Impostors path, if not `imposters`.
+    :param imposters_path: Imposters path, if not `imposters`.
 
     """
 
@@ -122,10 +123,10 @@ class MountebankServer:
         self.host = host
         self.scheme = scheme
         self.imposters_path = imposters_path
+        self._running_imposters: MutableSequence[Imposter] = []
 
     def __call__(self, imposters: Sequence[Imposter]) -> "MountebankServer":
         self.imposters = imposters
-        self.running_imposters_by_port = {}  # type: MutableMapping[int, Imposter]
         return self
 
     def __enter__(self) -> "MountebankServer":
@@ -148,23 +149,17 @@ class MountebankServer:
             json = definition.as_structure()
             post = requests.post(self.server_url, json=json, timeout=10)
             post.raise_for_status()
-            definition.port = post.json()["port"]
-            definition.host = self.host
-            self.running_imposters_by_port[cast(int, definition.port)] = definition
+            definition.attach(self.host, post.json()["port"], self.server_url)
+            self._running_imposters.append(definition)
 
     def delete_imposters(self) -> None:
-        while self.running_imposters_by_port:
-            imposter_port, imposter = self.running_imposters_by_port.popitem()
-            requests.delete(self.imposter_url(imposter_port)).raise_for_status()
+        while self._running_imposters:
+            imposter = self._running_imposters.pop()
+            requests.delete(imposter.configuration_url).raise_for_status()
 
-    def get_actual_requests(self) -> Mapping[int, JsonStructure]:
-        requests_by_impostor = {}
-        for imposter_port in self.running_imposters_by_port:
-            response = requests.get(self.imposter_url(imposter_port), timeout=5)
-            response.raise_for_status()
-            json = response.json()
-            requests_by_impostor[imposter_port] = json["requests"]
-        return requests_by_impostor
+    def get_actual_requests(self) -> Iterable[Request]:
+        for imposter in self._running_imposters:
+            yield from imposter.get_actual_requests()
 
     @property
     def server_url(self) -> furl:
@@ -172,10 +167,8 @@ class MountebankServer:
             scheme=self.scheme, host=self.host, port=self.server_port, path=self.imposters_path
         )
 
-    def imposter_url(self, imposter_port: int) -> furl:
-        return self.server_url.add(path=str(imposter_port))
-
-    def query_all_impostors(self):
+    def query_all_imposters(self) -> Iterator[Imposter]:
+        """Yield all imposters running on the server, including those defined elsewhere."""
         server_info = requests.get(self.server_url)
         imposters = server_info.json()["imposters"]
         for imposter in imposters:
@@ -183,7 +176,7 @@ class MountebankServer:
 
 
 class ExecutingMountebankServer(MountebankServer):
-    """A Mountebank mock server, running one or more impostors, one for each domain being mocked.
+    """A Mountebank mock server, running one or more imposters, one for each domain being mocked.
 
     Test will look like::
 
@@ -194,7 +187,7 @@ class ExecutingMountebankServer(MountebankServer):
                                 record_requests=True)
 
             with mb(imposter) as s:
-                r = requests.get('{0}/test'.format(imposter.url))
+                r = requests.get(f"{imposter.url}/test")
 
                 assert_that(r, is_response().with_status_code(200).and_body("sausages"))
                 assert_that(s, had_request(path='/test', method="GET"))
@@ -215,7 +208,7 @@ class ExecutingMountebankServer(MountebankServer):
     :param data_dir: Persist all operations to disk, in this directory.
     """
 
-    running = set()  # type: Set[int]
+    running: Set[int] = set()
     start_lock = Lock()
 
     def __init__(
@@ -228,12 +221,10 @@ class ExecutingMountebankServer(MountebankServer):
         local_only: bool = True,
         data_dir: Union[str, None] = ".mbdb",
     ) -> None:
-        super(ExecutingMountebankServer, self).__init__(port)
+        super().__init__(port)
         with self.start_lock:
             if self.server_port in self.running:
-                raise MountebankPortInUseException(
-                    "Already running on port {0}.".format(self.server_port)
-                )
+                raise MountebankPortInUseException(f"Already running on port {self.server_port}.")
             try:
                 options = self._build_options(port, debug, allow_injection, local_only, data_dir)
                 self.mb_process = subprocess.Popen([executable] + options)  # nosec
@@ -253,11 +244,7 @@ class ExecutingMountebankServer(MountebankServer):
     def _build_options(
         port: int, debug: bool, allow_injection: bool, local_only: bool, data_dir: Union[str, None]
     ):
-        options = [
-            "start",
-            "--port",
-            str(port),
-        ]  # type: List[str]
+        options: List[str] = ["start", "--port", str(port)]
         if debug:
             options.append("--debug")
         if allow_injection:
@@ -265,10 +252,7 @@ class ExecutingMountebankServer(MountebankServer):
         if local_only:
             options.append("--localOnly")
         if data_dir:
-            options += [
-                "--datadir",
-                data_dir,
-            ]
+            options += ["--datadir", data_dir]
         return options
 
     def _await_start(self, timeout: int) -> None:
@@ -284,9 +268,7 @@ class ExecutingMountebankServer(MountebankServer):
                 time.sleep(0.1)
 
         if not started:
-            raise MountebankTimeoutError(
-                "Mountebank failed to start within {0} seconds.".format(timeout)
-            )
+            raise MountebankTimeoutError(f"Mountebank failed to start within {timeout} seconds.")
 
         logger.debug("Server started at %s.", self.server_url)
 
@@ -305,16 +287,10 @@ class ExecutingMountebankServer(MountebankServer):
 class MountebankException(Exception):
     """Exception using Mountebank server."""
 
-    pass
-
 
 class MountebankPortInUseException(Exception):
     """Mountebank server failed to start - port already in use."""
 
-    pass
-
 
 class MountebankTimeoutError(MountebankException):
     """Mountebank server failed to start in time."""
-
-    pass
